//FRAGMENTSHADER_DIRECT_DEPTHSHADOW
precision mediump float;


uniform lowp vec4 u_directColor;
uniform sampler2D u_depthTex;
uniform float u_near;
uniform float u_far;
uniform vec3 u_lightDirection;


varying vec4 v_shadowCoord;
varying vec3 v_normal;


const vec4 bitSh = vec4(256.0*256.0*256.0, 256.0*256.0, 256.0, 1.0);
const vec4 bitMsk = vec4(0, 1.0/256.0, 1.0/256.0, 1.0/256.0);
const vec4 bitShifts = vec4(1.0 / (256.0 * 256.0 * 256.0), 1.0 / (256.0 * 256.0), 1.0 / 256.0, 1.0);
const vec2 poissonDisk1 = vec2(-0.94201624, -0.39906216);
const vec2 poissonDisk2 = vec2(0.94558609, -0.76890725);
const vec2 poissonDisk3 = vec2(-0.094184101, -0.92938870);
const vec2 poissonDisk4 = vec2(0.34495938, 0.29387760);


float unpack(vec4 color)
{
	return dot(color, bitShifts);
}
vec4 shadowFilter(vec2 depth, float fragDepth)
{
	float depthSQ = depth.x*depth.x;
	float variance = depth.y-depthSQ;
	float md = fragDepth-depth.x;
	float p = variance/(variance+(md*md));
	vec4 result;
	float f = clamp(p, 0.0, 1.0);
	f *= 0.5;
	if(depth.x > 0.0) {
		result = vec4(f, f, f, f*0.5+0.5);
//		result = vec4(1.0, 1.0, 1.0, 0.0);
	} else {
		result = vec4(f, f, f, 0.0);
//		result = vec4(1.0, 1.0, 1.0, 0.0);
	}
	return result;
}
float shadowESMFilter(float depth, float fragDepth)
{
	float C = -20.0;
	float result = 1.0;
	float distance = (fragDepth-depth);
    if(depth > 0.0) {
//        float f = exp(C*(distance));
//        if(f > 0.0) {
//            result = 0.92;
            result = 0.8;
//        }
    }
	return result;
}
float calcBias()
{
    float bias;
    float cosTheta = clamp( dot( v_normal, u_lightDirection), 0.0, 1.0);
    bias = 0.001*tan(acos(cosTheta));
    bias = clamp(bias, 0.0, 0.01);

    return bias;
}
void main()
{
////	vec4 shadowCoord = ((v_shadowCoord/v_shadowCoord.w)+1.0)*0.5;
////	vec2 depth = unpack2(texture2D(u_depthTex, shadowCoord.xy));
//////	vec4 depth = texture2D(u_depthTex, shadowCoord.xy);
////	float tdepth = (v_shadowCoord.w-near)/(far-near);
////	vec4 shadow = shadowFilter(depth, tdepth);
////	gl_FragColor=u_directColor*shadow;
//	vec4 shadowCoord = ((v_shadowCoord/v_shadowCoord.w)+1.0)*0.5;
//	float depth = unpack(texture2D(u_depthTex, shadowCoord.xy));
////	float tdepth = (v_shadowCoord.w-near)/(far-near);
//	float tdepth = shadowCoord.z;
//	vec4 shadow = shadowESMFilter(depth, tdepth);
//	if(shadow.w > 0.0) {
//    	gl_FragColor=shadow;
//    } else {
//        discard;
//    }

    float shadow = 1.0;
//    if(v_shadowCoord.w > 0.0) {
        vec4 shadowCoord = ((v_shadowCoord/v_shadowCoord.w)+1.0)*0.5;
        float bias = calcBias();
//        float tdepth = ((v_shadowCoord.w-u_near)/(u_far-u_near))-bias;
        float tdepth = shadowCoord.z;//-bias;
        shadow *= shadowESMFilter(unpack(texture2D(u_depthTex,shadowCoord.xy+poissonDisk1/1100.0)), tdepth);
        shadow *= shadowESMFilter(unpack(texture2D(u_depthTex,shadowCoord.xy+poissonDisk2/1100.0)), tdepth);
        shadow *= shadowESMFilter(unpack(texture2D(u_depthTex,shadowCoord.xy+poissonDisk3/1100.0)), tdepth);
        shadow *= shadowESMFilter(unpack(texture2D(u_depthTex,shadowCoord.xy+poissonDisk4/1100.0)), tdepth);
//    }
//    gl_FragColor=vec4(gl_FragColor.xyz*shadow, 1.0);
    if(shadow < 1.0)
    {
        gl_FragColor=vec4(shadow, shadow, shadow, 1.0);
    }
    else
    {
        discard;
    }
}
