//FRAGMENTSHADER_MQO_TEXTUREMUL
precision mediump float;

uniform sampler2D u_tex1;
uniform sampler2D u_tex2;

uniform vec3 u_lightDirection;
uniform vec4 u_materialAmbient;
uniform vec4 u_materialDiffuse;
uniform vec4 u_materialSpecular;
uniform float u_materialShininess;

varying vec3 v_position;
varying vec3 v_normal;
varying vec2 v_uv;


void main()
{
	vec3 normal = normalize(v_normal);
	vec3 cameraDirection = normalize(-v_position);
	float f = dot(u_lightDirection, normal)*0.5+0.5;
	vec4 diffuse = vec4(f, f, f, 1.0)*u_materialDiffuse;
	vec3 harfAngle = normalize(u_lightDirection + cameraDirection);
	float specularWeight = pow(max(0.001, dot(harfAngle, normal)), u_materialShininess);
	vec4 specular = specularWeight*u_materialSpecular;
	vec4 color = min(min(u_materialAmbient + diffuse, 1.0)*texture2D(u_tex1, v_uv)*texture2D(u_tex2, v_uv) + specular, 1.0);
	if(color.w > 0.0) {
		gl_FragColor = color;
	} else {
		discard;
	}
}
