//FRAGMENTSHADER_PRIMITIVE
precision mediump float;


uniform vec3 u_lightDirection;
uniform vec4 u_materialAmbient;
uniform vec4 u_materialDiffuse;
uniform vec4 u_materialSpecular;
uniform float u_materialShininess;
uniform sampler2D u_depthTex;


varying vec3 v_position;
varying vec3 v_normal;
varying vec4 v_shadowCoord;


const vec4 bitShifts = vec4(1.0 / (256.0 * 256.0 * 256.0), 1.0 / (256.0 * 256.0), 1.0 / 256.0, 1.0);
const vec2 poissonDisk1 = vec2(-0.94201624, -0.39906216);
const vec2 poissonDisk2 = vec2(0.94558609, -0.76890725);
const vec2 poissonDisk3 = vec2(-0.094184101, -0.92938870);
const vec2 poissonDisk4 = vec2(0.34495938, 0.29387760);
float unpack(vec4 color)
{
	return dot(color, bitShifts);
}
float shadowFilter(float depth, float fragDepth)
{
	float result = 1.0;
	float distance = (fragDepth-depth);
    if(distance > 0.0) {
	    result = 0.92;
    }
	return result;
}
float calcBias()
{
    float bias;
    float cosTheta = clamp( dot(v_normal, u_lightDirection), 0.0, 1.0);
    bias = 0.001*tan(acos(cosTheta));
    bias = clamp(bias, 0.0, 0.01);

    return bias;
}
void main()
{
	vec3 normal = normalize(v_normal);
	vec3 cameraDirection = normalize(-v_position);
	float f = dot(u_lightDirection, normal)*0.5+0.5;
	vec4 diffuse = vec4(f, f, f, 1.0)*u_materialDiffuse;
	vec3 harfAngle = normalize(u_lightDirection + cameraDirection);
	float specularWeight = pow(max(0.001, dot(harfAngle, normal)), u_materialShininess);
	vec4 specular = specularWeight*u_materialSpecular;

    float shadow = 1.0;
    if(v_shadowCoord.w > 0.0) {
        vec4 shadowCoord = ((v_shadowCoord/v_shadowCoord.w)+1.0)*0.5;
        float bias = calcBias();
        float tdepth = shadowCoord.z-bias;
        shadow *= shadowFilter(unpack(texture2D(u_depthTex,shadowCoord.xy+poissonDisk1/1100.0)), tdepth);
        shadow *= shadowFilter(unpack(texture2D(u_depthTex,shadowCoord.xy+poissonDisk2/1100.0)), tdepth);
        shadow *= shadowFilter(unpack(texture2D(u_depthTex,shadowCoord.xy+poissonDisk3/1100.0)), tdepth);
        shadow *= shadowFilter(unpack(texture2D(u_depthTex,shadowCoord.xy+poissonDisk4/1100.0)), tdepth);
    }
    vec4 ambDif = vec4((u_materialAmbient.xyz+u_materialDiffuse.xyz)*shadow, u_materialAmbient.w+u_materialDiffuse.w);
    vec4 color = min(min(ambDif, 1.0) + specular, 1.0);
	if(color.w > 0.0) {
		gl_FragColor = color;
	} else {
		discard;
	}
}
