//FRAGMENTSHADER_TEXTUREMULSPHEREANDTOON
precision mediump float;


uniform sampler2D u_tex1;
uniform sampler2D u_tex2;
uniform sampler2D u_toontex;
uniform lowp vec4 u_materialAmbient;
uniform lowp vec4 u_materialDiffuse;
uniform lowp vec4 u_materialSpecular;
uniform float u_materialShininess;
uniform vec3 u_lightDirection;
uniform float u_toonIntensity;


varying vec3 v_position;
varying vec3 v_normal;
varying vec2 v_uv;


void main()
{
	vec3 normal = normalize(v_normal);
	vec3 cameraDirection = normalize(-v_position);
	vec2 sphereCoord = 0.5*(1.0+vec2(1.0, -1.0)*normal.xy);
	
	vec4 texColor = texture2D(u_tex1, v_uv);
	texColor.xyz = texColor.xyz * texture2D(u_tex2, sphereCoord).xyz;
	
	vec3 harfAngle = normalize(u_lightDirection + cameraDirection);
	float specularWeight = pow(max(0.001, dot(harfAngle, normal)), u_materialShininess);
	vec4 specular = specularWeight*u_materialSpecular;
	vec4 color = min(min(u_materialAmbient+u_materialDiffuse, 1.0)*texColor + specular, 1.0);
	
    vec3 lightDir = normalize(u_lightDirection);
    float f = (dot(lightDir, normal)+1.0)*0.5;
//	float f = dot(u_lightDirection, normal)*0.5+0.5;
	vec2 uv = vec2(0.5, 1.0-f);
	vec3 t = vec3(1.0, 1.0, 1.0);
	vec3 toon = texture2D(u_toontex, uv).xyz;
	color.xyz *= toon+((t-toon)*(1.0-u_toonIntensity*f));
	
	if(color.w > 0.0) {
		gl_FragColor = color;
	} else {
		discard;
	}
}
